@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2013-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2013 Horde LLC
+ * @copyright 2013-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   HashTable
  */
@@ -17,7 +17,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2013 Horde LLC
+ * @copyright 2013-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   HashTable
  *
@@ -121,6 +121,14 @@ abstract class Horde_HashTable_Base implements ArrayAccess, Serializable
                 return false;
             }
 
+            if (!empty($this->_params['logger'])) {
+                $this->_params['logger']->debug(sprintf(
+                    '%s: Deleted keys (%s)',
+                    get_class($this),
+                    implode(',', array_keys($to_delete))
+                ));
+            }
+
             $this->_noexist = array_merge($this->_noexist, array_fill_keys(array_values($todo), true));
         }
 
@@ -191,7 +199,7 @@ abstract class Horde_HashTable_Base implements ArrayAccess, Serializable
      */
     protected function _getExists($keys, $callback)
     {
-        $out = $todo = array();
+        $noexist = $out = $todo = array();
 
         if (!($ret_array = is_array($keys))) {
             $keys = array($keys);
@@ -200,6 +208,7 @@ abstract class Horde_HashTable_Base implements ArrayAccess, Serializable
         foreach ($keys as $val) {
             if (isset($this->_noexist[$val])) {
                 $out[$val] = false;
+                $noexist[] = $val;
             } else {
                 $todo[$this->hkey($val)] = $val;
             }
@@ -209,15 +218,34 @@ abstract class Horde_HashTable_Base implements ArrayAccess, Serializable
             foreach (call_user_func($callback, array_keys($todo)) as $key => $val) {
                 if ($val === false) {
                     $this->_noexist[$todo[$key]] = true;
+                    $noexist[] = $todo[$key];
                 }
                 $out[$todo[$key]] = $val;
             }
         }
 
+        if (!empty($this->_params['logger'])) {
+            if ($tmp = array_diff(array_keys($out), $noexist)) {
+                $this->_params['logger']->debug(sprintf(
+                    '%s: Retrieved keys (%s)',
+                    get_class($this),
+                    implode(',', $tmp)
+                ));
+            }
+            if (!empty($noexist)) {
+                $this->_params['logger']->debug(sprintf(
+                    '%s: Non-existent keys (%s)',
+                    get_class($this),
+                    implode(',', $noexist)
+                ));
+            }
+        }
+
         return $ret_array
             ? $out
             : reset($out);
     }
+
     /**
      * Set the value of a key.
      *
@@ -225,11 +253,11 @@ abstract class Horde_HashTable_Base implements ArrayAccess, Serializable
      * @param string $val  The string to store.
      * @param array $opts  Additional options:
      * <pre>
+     *   - expire: (integer) Expiration time in seconds.
+     *             DEFAULT: Doesn't expire.
      *   - replace: (boolean) Replace the value of key. If key doesn't exist,
      *              returns false.
      *              DEFAULT: false
-     *   - timeout: (integer) Expiration time in seconds.
-     *              DEFAULT: Doesn't expire.
      * </pre>
      *
      * @return boolean  True on success, false on error.
@@ -240,12 +268,28 @@ abstract class Horde_HashTable_Base implements ArrayAccess, Serializable
             return false;
         }
 
-        if ($this->_set($this->hkey($key), $val, $opts)) {
-            unset($this->_noexist[$key]);
-            return true;
+        /* @todo BC: 'timeout' == 'expire' usage. */
+        if (isset($opts['timeout']) && !isset($opts['expire'])) {
+            $opts['expire'] = $opts['timeout'];
         }
 
-        return false;
+        if ($this->_set($this->hkey($key), $val, $opts)) {
+            unset($this->_noexist[$key]);
+            $res = true;
+        } else {
+            $res = false;
+        }
+
+        if (!empty($this->_params['logger'])) {
+            $this->_params['logger']->debug(sprintf(
+                '%s: Set key %s(%s)',
+                get_class($this),
+                $res ? '' : 'FAILED ',
+                $key
+            ));
+        }
+
+        return $res;
     }
 
     /**
