@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2013-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2013 Horde LLC
+ * @copyright 2013-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   HashTable
  */
@@ -18,7 +18,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2013 Horde LLC
+ * @copyright 2013-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   HashTable
  */
@@ -106,7 +106,17 @@ implements Horde_HashTable_Lock
         $keys = array_values($keys);
         $out = array();
 
-        foreach ($this->_predis->mget($keys) as $key => $val) {
+        try {
+            $data = $this->_predis->mget($keys);
+        } catch (Exception $e) {
+            /* MGET doesn't work on clusters. */
+            $data = array();
+            foreach ($keys as $key) {
+                $data[$key] = $this->_predis->get($key);
+            }
+        }
+
+        foreach ($data as $key => $val) {
             $out[$keys[$key]] = is_null($val)
                 ? false
                 : $val;
@@ -165,10 +175,10 @@ implements Horde_HashTable_Lock
     public function lock($key)
     {
         $hkey = $this->hkey($key) . self::LOCK_SUFFIX;
+        $i = 0;
 
         while (!$this->_predis->setnx($hkey, 1)) {
-            /* Wait 0.1 secs before attempting again. */
-            usleep(100000);
+            usleep(min(pow(2, $i++) * 10000, 100000));
         }
 
         $this->_predis->expire($hkey, self::LOCK_TIMEOUT);
@@ -180,7 +190,7 @@ implements Horde_HashTable_Lock
     public function unlock($key)
     {
         $this->_predis->del($this->hkey($key) . self::LOCK_SUFFIX);
-        $this->_locks[$key] = false;
+        unset($this->_locks[$key]);
     }
 
 }
